package com.sh.sdk.shareinstall_flutter;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;

import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;

import com.sh.sdk.shareinstall.ShareInstall;
import com.sh.sdk.shareinstall.listener.AppDeleteInstallParamsListener;
import com.sh.sdk.shareinstall.listener.AppGetInstallListener;
import com.sh.sdk.shareinstall.listener.AppGetWakeUpListener;
import com.sh.sdk.shareinstall.listener.OnReportRegisterListener;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * ShareinstallFlutterPlugin
 */
public class ShareinstallFlutterPlugin implements FlutterPlugin, MethodCallHandler, PluginRegistry.NewIntentListener {

    private static final String TAG = "shareInstall";

    private static final String GET_WAKEUP = "getWakeUp";
    private static final String GET_INSTALL = "getInstall";
    private static final String REPORT_REGISTER = "reportRegister";
    private static final String DEL_CUSTOM_PARAMS = "delCustomParams";

    private static ShareinstallFlutterPlugin instance;
    private MethodChannel methodChannel;
    private Context context;
    private final Object initializationLock = new Object();

    public ShareinstallFlutterPlugin() {
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        if (instance == null) {
            instance = new ShareinstallFlutterPlugin();
        }
        instance.onAttachedToEngine(registrar.context(), registrar.messenger());
        registrar.addNewIntentListener(instance);
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        String method = call.method;
        if (method.equals(GET_WAKEUP)) {
            getWakeUp(call, result);
            return;
        }
        if (method.equals(GET_INSTALL)) {
            getInstall(call, result);
            return;
        }
        if (method.equals(REPORT_REGISTER)) {
            reportRegister(call, result);
            return;
        }
        if (method.equals(DEL_CUSTOM_PARAMS)) {
            delCustomParams(result);
            return;
        }
        result.notImplemented();
    }

    private void getWakeUp(MethodCall call, final Result result) {
        //Log.e(TAG, "getWakeUp");
        String data = (String) call.arguments;
        Intent intent = new Intent();
        if (!TextUtils.isEmpty(data)) {
            intent.setData(Uri.parse(data));
        }
        ShareInstall.getInstance().getWakeUpParams(intent, new AppGetWakeUpListener() {
            @Override
            public void onGetWakeUpFinish(String json) {
                result.success(json);
            }
        });
    }

    private void getWakeUp(Intent intent) {
        //Log.e(TAG, "getWakeUp onNewIntent");
        ShareInstall.getInstance().getWakeUpParams(intent, new AppGetWakeUpListener() {
            @Override
            public void onGetWakeUpFinish(String json) {
                methodChannel.invokeMethod("wakeup", json);
            }
        });
    }


    private void getInstall(MethodCall call, final Result result) {
        //Log.e(TAG, "getInstall");
        ShareInstall.getInstance().getInstallParams(new AppGetInstallListener() {
            @Override
            public void onGetInstallFinish(String json) {
                result.success(json);
            }
        });
    }

    private void reportRegister(MethodCall call, final Result result) {
        //Log.e(TAG, "reportRegister");
        ShareInstall.getInstance().reportRegister(new OnReportRegisterListener() {
            @Override
            public void onSuccess() {
                result.success("reportRegister success");
            }

            @Override
            public void onError() {
                result.success("reportRegister error");
            }
        });
    }

    private void delCustomParams(final Result result) {
        //Log.e(TAG, "delCustomParams");
        ShareInstall.getInstance().deleteInstallParams(new AppDeleteInstallParamsListener() {
            @Override
            public void onDeleteInstallParamsComplete() {
                result.success("delCustomParams complete");
            }
        });
    }

    @Override
    public boolean onNewIntent(Intent intent) {
        //Log.e(TAG, "onNewIntent");
        getWakeUp(intent);
        return false;
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        onAttachedToEngine(binding.getApplicationContext(), binding.getBinaryMessenger());
    }

    private void onAttachedToEngine(Context applicationContext, BinaryMessenger messenger) {
        synchronized (initializationLock) {
            if (methodChannel != null) {
                return;
            }
            //Log.e(TAG, "onAttachedToEngine");
            this.context = applicationContext;
            methodChannel = new MethodChannel(messenger, "shareinstall_flutter");
            methodChannel.setMethodCallHandler(this);
            ShareInstall.getInstance().init(context);
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        context = null;
        methodChannel.setMethodCallHandler(null);
        methodChannel = null;
    }
}
